﻿using System;
using SDX.Compiler;
using Mono.Cecil;
using Mono.Cecil.Cil;
using System.Linq;
using System.Collections.Generic;

public class PlayerSignPrefabMessage : IPatcherMod
{
    public bool Link(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        InsertCW(gameModule, modModule);
        InsertOW(gameModule, modModule);
        InsertSetTileEntitySign(gameModule, modModule);
        InsertCopyPrefabData(gameModule, modModule);
        return true;
    }
    public void InsertCopyPrefabData(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "Prefab");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == ".ctor" && d.Parameters.Count == 1);
        ILProcessor proc1 = methodDef.Body.GetILProcessor();
        Instruction insertPos = methodDef.Body.Instructions[0];

        TypeDefinition playerSignPrefabMessageMethod = modModule.Types.First(d => d.Name == "PlayerSignPrefabMessageMethod");
        MethodDefinition copyPrefabData = playerSignPrefabMessageMethod.Methods.First(d => d.Name == "CopyPrefabData");
        MethodReference copyPrefabDataRef = gameModule.Import(copyPrefabData);
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_1));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, copyPrefabDataRef));

    }
    public void InsertSetTileEntitySign(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "Prefab");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "CopyBlocksIntoChunk" && d.Parameters.Count == 4);

        ILProcessor proc1 = methodDef.Body.GetILProcessor();

        Instruction insertPos = null;
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Callvirt && inst.Operand.GetType() == typeof(MethodDefinition) && ((MethodDefinition)inst.Operand).FullName == "BlockValue Chunk::SetBlock(WorldBase,System.Int32,System.Int32,System.Int32,BlockValue)")
            {
                insertPos = inst.Next.Next.Next.Next;
            }
        }
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_1));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_2));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[16]));//BlockValue
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[10]));//i
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_3));//Num2
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Add));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[15]));//k
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[11]));//j
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[7]));//Num6
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Add));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[12]));//x
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[17]));//Num10
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, methodDef.Body.Variables[13]));//z

        TypeDefinition playerSignPrefabMessageMethod = modModule.Types.First(d => d.Name == "PlayerSignPrefabMessageMethod");
        MethodDefinition getTileEntitySign = playerSignPrefabMessageMethod.Methods.First(d => d.Name == "SetTileEntitySign");
        MethodReference getTileEntitySignRef = gameModule.Import(getTileEntitySign);
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getTileEntitySignRef));

    }

    public void InsertCW(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "Prefab");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "CW");

        TypeDefinition playerSignPrefabMessageMethod = modModule.Types.First(d => d.Name == "PlayerSignPrefabMessageMethod");
        MethodDefinition getPlayerSignProperty = playerSignPrefabMessageMethod.Methods.First(d => d.Name == "GetPlayerSignProperty");
        MethodReference getPlayerSignPropertyRef = gameModule.Import(getPlayerSignProperty);


        ILProcessor proc1 = methodDef.Body.GetILProcessor();

        Instruction insertPos = methodDef.Body.Instructions[0];

        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getPlayerSignPropertyRef));

    }

    public void InsertOW(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "Prefab");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "OW");

        TypeDefinition playerSignPrefabMessageMethod = modModule.Types.First(d => d.Name == "PlayerSignPrefabMessageMethod");
        MethodDefinition getPlayerSignDataToProperty = playerSignPrefabMessageMethod.Methods.First(d => d.Name == "GetPlayerSignDataToProperty");
        MethodReference getPlayerSignDataToPropertyRef = gameModule.Import(getPlayerSignDataToProperty);

        ILProcessor proc1 = methodDef.Body.GetILProcessor();

        Instruction insertPos = methodDef.Body.Instructions[0];

        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getPlayerSignDataToPropertyRef));

    }

    public bool Patch(ModuleDefinition module)
    {
        Console.WriteLine("==PlayerSignPrefabMessage Patcher===");
        AddPrefabClassField(module);
        PrefabHWFieldPublic(module);
        return true;
    }
    public void AddPrefabClassField(ModuleDefinition module)
    {
        TypeDefinition prefabClass = module.Types.First(d => d.Name == "Prefab");
        TypeReference dictoinaryType = module.Import(typeof(System.Collections.Generic.Dictionary<,>));
        TypeReference objectRef = module.Import(typeof(object));
        TypeReference intRef = module.Import(typeof(int));
        GenericInstanceType giDictionaryType1 = new GenericInstanceType(dictoinaryType);
        giDictionaryType1.GenericArguments.Add(intRef);
        giDictionaryType1.GenericArguments.Add(objectRef);
        AddFieldDefinition(prefabClass, "TileEntityPlayerDic", FieldAttributes.Public, giDictionaryType1);

    }

    public void PrefabHWFieldPublic(ModuleDefinition module)
    {
        TypeDefinition prefabClass = module.Types.First(d => d.Name == "Prefab");
        FieldDefinition field = prefabClass.Fields.First(d => d.Name == "HW");
        SetFieldToPublic(field);
    }

    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        classDef.Fields.Add(fd);

    }
    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType, object constantvalue)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        fd.Constant = constantvalue;
        classDef.Fields.Add(fd);

    }
    private void SetFieldToPublic(FieldDefinition field)
    {
        field.IsFamily = false;
        field.IsPrivate = false;
        field.IsPublic = true;

    }

}
