﻿using System;
using System.Linq;
using System.Xml;
using System.Collections.Generic;
using UnityEngine;
using System.Text;

public static class PlayerSignPrefabMessageMethod
{
    public class PrefabSignMessageData
    {
        public PrefabSignMessageData()
        {

        }
        public PrefabSignMessageData(int x, int y, int z, string message)
        {
            _x = x;
            _y = y;
            _z = z;
            _message = message;
        }
        public PrefabSignMessageData Clone()
        {
            return new PrefabSignMessageData(_x, _y, _z, _message);
        }
        private int _x;
        public int X
        {
            get
            {
                return _x;
            }
            set
            {
                _x = value;
            }
        }
        private int _y;
        public int Y
        {
            get
            {
                return _y;
            }
            set
            {
                _y = value;
            }
        }
        private int _z;
        public int Z
        {
            get
            {
                return _z;
            }
            set
            {
                _z = value;
            }
        }
        private string _message;
        public string Message
        {
            get
            {
                return _message;
            }
            set
            {
                _message = value;
            }
        }
    }

    //Prefab.OW
    public static void GetPlayerSignDataToProperty(Prefab prefab)
    {
        if (prefab.TileEntityPlayerDic != null)
        {
            StringBuilder sbPoistion = new StringBuilder();
            StringBuilder sbMessage = new StringBuilder();
            foreach (object value in prefab.TileEntityPlayerDic.Values)
            {
                PrefabSignMessageData prefabSignMessageData = (PrefabSignMessageData)value;
                if (sbPoistion.Length > 0)
                {
                    sbPoistion.Append('#');

                }
                sbPoistion.Append(prefabSignMessageData.X);
                sbPoistion.Append(',');
                sbPoistion.Append(prefabSignMessageData.Y);
                sbPoistion.Append(',');
                sbPoistion.Append(prefabSignMessageData.Z);
                if(sbMessage.Length > 0)
                {
                    sbMessage.Append("###");
                }
                sbMessage.Append(prefabSignMessageData.Message);
            }
            prefab.properties.Values["PlayerSignPosition"] = sbPoistion.ToString();
            prefab.properties.Values["PlayerSignMessage"] = sbMessage.ToString();

        }

    }
    //Prefab.CW
    public static void GetPlayerSignProperty(Prefab prefab)
    {
        if (prefab.properties.Values.ContainsKey("PlayerSignPosition"))
        {
            if (prefab.properties.Values.ContainsKey("PlayerSignMessage"))
            {

                string playerSignPosition = prefab.properties.GetStringValue("PlayerSignPosition");
                string[] positionArray = playerSignPosition.Split(new char[] { '#' }, StringSplitOptions.RemoveEmptyEntries);
                string playerSignMessage = prefab.properties.GetStringValue("PlayerSignMessage");
                string[] messageArray = playerSignMessage.Split(new string[] {"###" }, StringSplitOptions.RemoveEmptyEntries);
                for (int i = 0; i < positionArray.Length; i++)
                {
                    string[] array = positionArray[i].Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                    int x = int.Parse(array[0]);
                    int y = int.Parse(array[1]);
                    int z = int.Parse(array[2]);
                    int num = FW(x, y, z, prefab, 0);
                    if (prefab.TileEntityPlayerDic == null)
                    {
                        prefab.TileEntityPlayerDic = new Dictionary<int,  object>();
                    }
                    prefab.TileEntityPlayerDic[num] = new PrefabSignMessageData(x, y, z, messageArray[i]);
                }
            }
        }
    }
    public static bool CheckOwnerExist(TileEntitySign sign)
    {
        return (sign.GetOwner() != string.Empty);
    }
    public static void SetTileEntitySign(World world, Chunk chunk, Prefab prefab, BlockValue blockValue, int prefabX, int prefabY, int prefabZ, int chunkX, int chunkY, int chunkZ)
    {
        int blockID = blockValue.type;
        if (blockID == 886 || blockID == 887 || blockID == 888)
        {
            if (prefab.TileEntityPlayerDic != null)
            {
                int num = FW(prefabX, prefabY, prefabZ, prefab, prefab.HW);
                if (prefab.TileEntityPlayerDic.ContainsKey(num))
                {
                    Vector3i blockPos = new Vector3i(chunkX, chunkY, chunkZ);
                    BlockEntityData bed = chunk.GetBlockEntity(blockPos);


                    global::TileEntitySign tileEntitySign = new global::TileEntitySign(chunk);
                    if (tileEntitySign != null)
                    {
                        tileEntitySign.localChunkPos = global::World.toBlock(blockPos);
                        chunk.AddTileEntity(tileEntitySign);
                    }
                    tileEntitySign.SetBlockEntityData(bed);

                    tileEntitySign.SetText(((PrefabSignMessageData)prefab.TileEntityPlayerDic[num]).Message,true);
                }
            }
        }
    }
    public static void CopyPrefabData(Prefab src,Prefab dst)
    {
        if(src.TileEntityPlayerDic == null)
        {
            dst.TileEntityPlayerDic = null;
        }
        else
        {
            dst.TileEntityPlayerDic = new Dictionary<int, object>();
            foreach (KeyValuePair<int,object> keyValue in src.TileEntityPlayerDic)
            {
                dst.TileEntityPlayerDic.Add(keyValue.Key,((PrefabSignMessageData)keyValue.Value).Clone());
            }
        }
    }
    private static int FW(int A_1, int A_2, int A_3, Prefab prefab,int hw)
    {
        int result = 0;
        switch (hw)
        {
            case 0:
                result = A_1 + A_2 * prefab.size.x + A_3 * prefab.size.x * prefab.size.y;
                break;
            case 1:
                result = A_3 + A_2 * prefab.size.z + (prefab.size.x - A_1 - 1) * prefab.size.z * prefab.size.y;
                break;
            case 2:
                result = prefab.size.x - A_1 - 1 + A_2 * prefab.size.x + (prefab.size.z - A_3 - 1) * prefab.size.x * prefab.size.y;
                break;
            case 3:
                result = prefab.size.z - A_3 - 1 + A_2 * prefab.size.z + A_1 * prefab.size.z * prefab.size.y;
                break;
            default:
                result = A_1 + A_2 * prefab.size.x + A_3 * prefab.size.x * prefab.size.y;
                break;
        }
        return result;
    }


}
